@@ -473,16 +473,6 @@ then
     cp -f drivers/human/car-corvette.xml $1/drivers/human/car-corvette.xml
     chmod 640 $1/drivers/human/car-corvette.xml
 fi
-if [ ! -e  $1/drivers/human/car-fs-02h2.xml ] || [ drivers/human/car-fs-02h2.xml -nt $1/drivers/human/car-fs-02h2.xml ]
-then
-    if [ -e $1/drivers/human/car-fs-02h2.xml ]
-    then
-        echo "Saving $1/drivers/human/car-fs-02h2.xml to $1/drivers/human/car-fs-02h2.xml.old"
-        cp -f $1/drivers/human/car-fs-02h2.xml $1/drivers/human/car-fs-02h2.xml.old
-    fi
-    cp -f drivers/human/car-fs-02h2.xml $1/drivers/human/car-fs-02h2.xml
-    chmod 640 $1/drivers/human/car-fs-02h2.xml
-fi
 if [ ! -e  $1/drivers/human/car-lotus-gt1.xml ] || [ drivers/human/car-lotus-gt1.xml -nt $1/drivers/human/car-lotus-gt1.xml ]
 then
     if [ -e $1/drivers/human/car-lotus-gt1.xml ]
@@ -543,16 +533,6 @@ then
     cp -f drivers/human/car-pw-206wrc.xml $1/drivers/human/car-pw-206wrc.xml
     chmod 640 $1/drivers/human/car-pw-206wrc.xml
 fi
-if [ ! -e  $1/drivers/human/car-pw-evovwrc-k.xml ] || [ drivers/human/car-pw-evovwrc-k.xml -nt $1/drivers/human/car-pw-evovwrc-k.xml ]
-then
-    if [ -e $1/drivers/human/car-pw-evovwrc-k.xml ]
-    then
-        echo "Saving $1/drivers/human/car-pw-evovwrc-k.xml to $1/drivers/human/car-pw-evovwrc-k.xml.old"
-        cp -f $1/drivers/human/car-pw-evovwrc-k.xml $1/drivers/human/car-pw-evovwrc-k.xml.old
-    fi
-    cp -f drivers/human/car-pw-evovwrc-k.xml $1/drivers/human/car-pw-evovwrc-k.xml
-    chmod 640 $1/drivers/human/car-pw-evovwrc-k.xml
-fi
 if [ ! -e  $1/drivers/human/car-pw-evovwrc.xml ] || [ drivers/human/car-pw-evovwrc.xml -nt $1/drivers/human/car-pw-evovwrc.xml ]
 then
     if [ -e $1/drivers/human/car-pw-evovwrc.xml ]
@@ -655,6 +635,16 @@ then
     cp -f config/graph.xml $1/config/graph.xml
     chmod 640 $1/config/graph.xml
 fi
+if [ ! -e  $1/config/sound.xml ] || [ config/sound.xml -nt $1/config/sound.xml ]
+then
+    if [ -e $1/config/sound.xml ]
+    then
+        echo "Saving $1/config/sound.xml to $1/config/sound.xml.old"
+        cp -f $1/config/sound.xml $1/config/sound.xml.old
+    fi
+    cp -f config/sound.xml $1/config/sound.xml
+    chmod 640 $1/config/sound.xml
+fi
 mkdir -p $1/results/champ 2>/dev/null
 mkdir -p $1/results/dtmrace 2>/dev/null
 mkdir -p $1/results/endrace 2>/dev/null
