@@ -4,7 +4,7 @@
     created              : Sun Jan 30 12:00:15 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: car.h,v 1.32 2004/11/28 18:53:30 olethros Exp $
+    version              : $Id: car.h,v 1.35 2005/07/19 19:55:52 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file
     		This is the car structure.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: car.h,v 1.32 2004/11/28 18:53:30 olethros Exp $
+    @version	$Id: car.h,v 1.35 2005/07/19 19:55:52 berniw Exp $
     @ingroup	carstruct
     @note	Short cuts are to be used with the carElt structure.
 */
@@ -509,7 +509,10 @@ typedef struct CarElt
 #define PRM_FUELCONS		"fuel cons factor"
 #define PRM_ENGBRKCOEFF		"brake coefficient"
 #define PRM_POWER		"power"
-
+#define PRM_TURBO               "turbo"
+#define PRM_TURBO_RPM           "turbo rpm"
+#define PRM_TURBO_FACTOR        "turbo factor"
+#define PRM_TURBO_LAG           "turbo lag"
 
 #define PRM_RATIO		"ratio"
 #define PRM_BIAS		"bias"
@@ -591,7 +594,17 @@ typedef struct CarElt
 #define PRM_DAMAGE_SUSPENSION "damage/suspension"
 #define PRM_DAMAGE_ALIGNMENT "damage/alignment"
 #define PRM_DAMAGE_AERO "damage/aero"
-#define PRM_MODEL_AEROFLOW "model/aeroflow"
+#define PRM_MODEL_AEROFLOW "model/aero/flow"
+#define PRM_MODEL_AERO_FACTOR "model/aero/factor"
+#define PRM_MODEL_TYRE_TEMPERATURE "model/tyre/temperature"
+
+// Collision constants.
+#define SEM_COLLISION			0x01
+#define SEM_COLLISION_XYSCENE	0x02
+#define SEM_COLLISION_CAR		0x04
+#define SEM_COLLISION_Z			0x08
+#define SEM_COLLISION_Z_CRASH	0x10
+
 
 #endif /* __CARV1_H__ */ 
 
