@@ -4,7 +4,7 @@
     created              : Sun Jan 30 12:02:05 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: simu.h,v 1.6 2004/10/07 17:05:04 olethros Exp $
+    version              : $Id: simu.h,v 1.7 2005/07/19 19:55:52 berniw Exp $
 
  ***************************************************************************/
 
@@ -29,7 +29,7 @@
 struct Situation;
 struct RmInfo;
 
-typedef void (*tfSimInit)(int);
+typedef void (*tfSimInit)(int, tTrack*);
 typedef void (*tfSimConfig)(tCarElt*, struct RmInfo*);
 typedef void (*tfSimReConfig)(tCarElt*);
 typedef void (*tfSimUpdate)(struct Situation*, double, int);
