@@ -4,7 +4,7 @@
     created              : Sun Jan 30 23:00:06 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: track.h,v 1.19 2004/10/07 17:05:04 olethros Exp $
+    version              : $Id: track.h,v 1.21 2005/08/05 08:57:21 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file
     		This is the track structure.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: track.h,v 1.19 2004/10/07 17:05:04 olethros Exp $
+    @version	$Id: track.h,v 1.21 2005/08/05 08:57:21 berniw Exp $
     @ingroup	trackstruct
 */
 
@@ -29,6 +29,7 @@
 #define _TRACKV1_H_
 
 #include <tgf.h>
+#include <tmath/linalg_t.h>
 
 #define TRK_IDENT	0	/* from 0x01 to 0xFF */
 
@@ -251,6 +252,7 @@ typedef struct trackBarrier {
     tdble		width;	/**< Barrier width */
     tdble		height;	/**< Barrier height */
     tTrackSurface	*surface; /**< Barrier surface */
+	vec2f normal;	// Normal on the vertical track inside pointing towards the track middle.
 } tTrackBarrier;
 
 
@@ -381,12 +383,21 @@ typedef struct trackSeg {
     tSegExt		*ext;
 
     tTrackSurface	*surface; /**< Segment surface */
-    tTrackBarrier	*barrier[2]; /**< Segment barriers */
+	tTrackBarrier	*barrier[2]; /**< Segment barriers */
     tRoadCam        *cam;	/* current camera */
     struct trackSeg *next;	/**< Next segment */
     struct trackSeg *prev;	/**< Previous segment */
-    struct trackSeg *lside;	/**< Segment on the left */
-    struct trackSeg *rside;	/**< Segment on the right */
+
+	// Union to avoid code duplication for left/right side cases and to
+	// keep compatibility of code. The side definition is so ugly to
+	// match the one of the barrier[].
+#define TR_SIDE_LFT 1
+#define TR_SIDE_RGT 0
+	union {
+		struct { struct trackSeg *rside, *lside; };
+		struct trackSeg* side[2];
+	};
+
 } tTrackSeg;
 
 /* selection for local position structure */
