@@ -4,7 +4,7 @@
     created              : Sat Mar 18 23:41:55 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: entry.cpp,v 1.2 2003/06/24 21:02:23 torcs Exp $
+    version              : $Id: entry.cpp,v 1.4 2005/02/01 15:55:51 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,6 +20,7 @@
 
 #include <client.h>
 #include <tgfclient.h>
+#include <plib/ssg.h>
 
 #include "mainmenu.h"
 #include "splash.h"
@@ -43,6 +44,8 @@
 void
 TorcsEntry(void)
 {
+    ssgInit();
+
     GfInitClient();
 
     TorcsMainMenuInit();
