@@ -4,7 +4,7 @@
     created     : Sat Nov 23 09:05:23 CET 2002
     copyright   : (C) 2002 by Eric Espié                        
     email       : eric.espie@torcs.org   
-    version     : $Id: raceengine.cpp,v 1.13 2005/02/01 15:55:51 berniw Exp $                                  
+    version     : $Id: raceengine.cpp,v 1.16 2005/08/17 20:48:39 berniw Exp $                                  
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file   
     		
     @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
-    @version	$Id: raceengine.cpp,v 1.13 2005/02/01 15:55:51 berniw Exp $
+    @version	$Id: raceengine.cpp,v 1.16 2005/08/17 20:48:39 berniw Exp $
 */
 
 #include <stdlib.h>
@@ -51,64 +51,64 @@ static void ReRaceRules(tCarElt *car);
 static void
 ReUpdtPitTime(tCarElt *car)
 {
-    tSituation	*s = ReInfo->s;
-    tReCarInfo	*info = &(ReInfo->_reCarInfo[car->index]);
-    int		i;
+	tSituation *s = ReInfo->s;
+	tReCarInfo *info = &(ReInfo->_reCarInfo[car->index]);
+	int i;
 
-    switch (car->_pitStopType) {
-    case RM_PIT_REPAIR:
-	info->totalPitTime = 2.0 + fabs(car->_pitFuel) / 8.0 + (tdble)(fabs(car->_pitRepair)) * 0.007;
-	car->_scheduledEventTime = s->currentTime + info->totalPitTime;
-	ReInfo->_reSimItf.reconfig(car);
-	for (i=0; i<4; i++) {
-		car->_tyreCondition(i) = 1.01;
-		car->_tyreT_in(i) = 50.0;
-		car->_tyreT_mid(i) = 50.0;
-		car->_tyreT_out(i) = 50.0;
+	switch (car->_pitStopType) {
+		case RM_PIT_REPAIR:
+			info->totalPitTime = 2.0f + fabs(car->_pitFuel) / 8.0f + (tdble)(fabs(car->_pitRepair)) * 0.007f;
+			car->_scheduledEventTime = s->currentTime + info->totalPitTime;
+			ReInfo->_reSimItf.reconfig(car);
+			for (i=0; i<4; i++) {
+				car->_tyreCondition(i) = 1.01;
+				car->_tyreT_in(i) = 50.0;
+				car->_tyreT_mid(i) = 50.0;
+				car->_tyreT_out(i) = 50.0;
+			}
+			break;
+		case RM_PIT_STOPANDGO:
+			info->totalPitTime = 0.0;
+			car->_scheduledEventTime = s->currentTime;
+			break;
 	}
-	break;
-    case RM_PIT_STOPANDGO:
-	info->totalPitTime = 0.0;
-	car->_scheduledEventTime = s->currentTime;
-	break;
-    }
 }
 
 /* Return from interactive pit information */
 static void
 ReUpdtPitCmd(void *pvcar)
 {
-    tCarElt *car = (tCarElt*)pvcar;
-    
-    ReUpdtPitTime(car);
-    //ReStart(); /* resynchro */
-    GfuiScreenActivate(ReInfo->_reGameScreen);
+	tCarElt *car = (tCarElt*)pvcar;
+
+	ReUpdtPitTime(car);
+	//ReStart(); /* resynchro */
+	GfuiScreenActivate(ReInfo->_reGameScreen);
 }
 
 static void
 ReRaceMsgUpdate(void)
 {
-    if (ReInfo->_reCurTime > msgDisp) {
-	ReSetRaceMsg("");
-    }
-    if (ReInfo->_reCurTime > bigMsgDisp) {
-	ReSetRaceBigMsg("");
-    }
+	if (ReInfo->_reCurTime > msgDisp) {
+		ReSetRaceMsg("");
+	}
+	if (ReInfo->_reCurTime > bigMsgDisp) {
+		ReSetRaceBigMsg("");
+	}
 }
 
 static void
 ReRaceMsgSet(char *msg, double life)
 {
-    ReSetRaceMsg(msg);
-    msgDisp = ReInfo->_reCurTime + life;
+	ReSetRaceMsg(msg);
+	msgDisp = ReInfo->_reCurTime + life;
 }
 
 
 static void
 ReRaceBigMsgSet(char *msg, double life)
 {
-    ReSetRaceBigMsg(msg);
-    bigMsgDisp = ReInfo->_reCurTime + life;
+	ReSetRaceBigMsg(msg);
+	bigMsgDisp = ReInfo->_reCurTime + life;
 }
 
 
@@ -431,13 +431,11 @@ ReRaceRules(tCarElt *car)
 		    /* it's not more a drive through */
 		    rules->ruleState &= ~RM_PNST_DRIVETHROUGH;
 		} else if (rules->ruleState & RM_PNST_STOPANDGO) {
-		    if (car->_pitStopType == RM_PIT_STOPANDGO) {
-			/* the driver wants to clean the stop and go */
-			rules->ruleState |= RM_PNST_STOPANDGO_OK;
-		    } else {
-			/* a refuel pit stop does not count as stop and go */
-			rules->ruleState &= ~RM_PNST_STOPANDGO;
-		    }
+		    rules->ruleState |= RM_PNST_STOPANDGO_OK;
+		}
+	    } else {
+                if(rules->ruleState & RM_PNST_STOPANDGO_OK && car->_pitStopType != RM_PIT_STOPANDGO) {
+		    rules->ruleState &= ~ ( RM_PNST_STOPANDGO | RM_PNST_STOPANDGO_OK );
 		}
 	    }
 	} else if (seg->raceInfo & TR_PITEND) {
@@ -499,52 +497,52 @@ ReRaceRules(tCarElt *car)
 static void
 ReOneStep(double deltaTimeIncrement)
 {
-    int i;
-    tRobotItf *robot;
-    tSituation	*s = ReInfo->s;
+	int i;
+	tRobotItf *robot;
+	tSituation *s = ReInfo->s;
 
-    if (floor(s->currentTime) == -2.0) {
-	ReRaceBigMsgSet("Ready", 1.0);
-    } else if (floor(s->currentTime) == -1.0) {
-	ReRaceBigMsgSet("Set", 1.0);
-    } else if (floor(s->currentTime) == 0.0) {
-	ReRaceBigMsgSet("Go", 1.0);
-    }
-
-    ReInfo->_reCurTime += deltaTimeIncrement * ReInfo->_reTimeMult; /* "Real" time */
-    s->currentTime += deltaTimeIncrement; /* Simulated time */
-
-    if (s->currentTime < 0) {
-	/* no simu yet */
-	ReInfo->s->_raceState = RM_RACE_PRESTART;
-    } else if (ReInfo->s->_raceState == RM_RACE_PRESTART) {
-	ReInfo->s->_raceState = RM_RACE_RUNNING;
-	s->currentTime = 0.0; /* resynchronize */
-	ReInfo->_reLastTime = 0.0;
-    }
-    
-    START_PROFILE("rbDrive*");
-    if ((s->currentTime - ReInfo->_reLastTime) >= RCM_MAX_DT_ROBOTS) {
-	s->deltaTime = s->currentTime - ReInfo->_reLastTime;
-	for (i = 0; i < s->_ncars; i++) {
-	    if ((s->cars[i]->_state & RM_CAR_STATE_NO_SIMU) == 0) {
-		robot = s->cars[i]->robot;
-		robot->rbDrive(robot->index, s->cars[i], s);
-	    }
+	if (floor(s->currentTime) == -2.0) {
+		ReRaceBigMsgSet("Ready", 1.0);
+	} else if (floor(s->currentTime) == -1.0) {
+		ReRaceBigMsgSet("Set", 1.0);
+	} else if (floor(s->currentTime) == 0.0) {
+		ReRaceBigMsgSet("Go", 1.0);
 	}
-	ReInfo->_reLastTime = s->currentTime;
-    }
-    STOP_PROFILE("rbDrive*");
 
-    START_PROFILE("_reSimItf.update*");
-    ReInfo->_reSimItf.update(s, deltaTimeIncrement, -1);
-    for (i = 0; i < s->_ncars; i++) {
-	ReManage(s->cars[i]);
-    }
-    STOP_PROFILE("_reSimItf.update*");
+	ReInfo->_reCurTime += deltaTimeIncrement * ReInfo->_reTimeMult; /* "Real" time */
+	s->currentTime += deltaTimeIncrement; /* Simulated time */
 
-    ReRaceMsgUpdate();
-    ReSortCars();
+	if (s->currentTime < 0) {
+		/* no simu yet */
+		ReInfo->s->_raceState = RM_RACE_PRESTART;
+	} else if (ReInfo->s->_raceState == RM_RACE_PRESTART) {
+		ReInfo->s->_raceState = RM_RACE_RUNNING;
+		s->currentTime = 0.0; /* resynchronize */
+		ReInfo->_reLastTime = 0.0;
+	}
+
+	START_PROFILE("rbDrive*");
+	if ((s->currentTime - ReInfo->_reLastTime) >= RCM_MAX_DT_ROBOTS) {
+		s->deltaTime = s->currentTime - ReInfo->_reLastTime;
+		for (i = 0; i < s->_ncars; i++) {
+			if ((s->cars[i]->_state & RM_CAR_STATE_NO_SIMU) == 0) {
+				robot = s->cars[i]->robot;
+				robot->rbDrive(robot->index, s->cars[i], s);
+			}
+		}
+		ReInfo->_reLastTime = s->currentTime;
+	}
+	STOP_PROFILE("rbDrive*");
+
+	START_PROFILE("_reSimItf.update*");
+	ReInfo->_reSimItf.update(s, deltaTimeIncrement, -1);
+	for (i = 0; i < s->_ncars; i++) {
+		ReManage(s->cars[i]);
+	}
+	STOP_PROFILE("_reSimItf.update*");
+
+	ReRaceMsgUpdate();
+	ReSortCars();
 }
 
 void
