@@ -4,7 +4,7 @@
     created              : Fri Aug 13 22:01:33 CEST 1999
     copyright            : (C) 1999 by Eric Espie                         
     email                : torcs@free.fr   
-    version              : $Id: gui.cpp,v 1.4 2005/02/01 15:55:54 berniw Exp $                                  
+    version              : $Id: gui.cpp,v 1.5 2005/08/05 09:26:39 berniw Exp $                                  
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file
     		This API is used to manage all the menu screens.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: gui.cpp,v 1.4 2005/02/01 15:55:54 berniw Exp $
+    @version	$Id: gui.cpp,v 1.5 2005/08/05 09:26:39 berniw Exp $
     @ingroup	gui
 */
 
@@ -500,6 +500,8 @@ GfuiScreenDeactivate(void)
 
     glutKeyboardFunc((void(*)(unsigned char,int,int))NULL);
     glutSpecialFunc((void(*)(int,int,int))NULL);
+	glutKeyboardUpFunc((void(*)(unsigned char,int,int))NULL);
+    glutSpecialUpFunc((void(*)(int,int,int))NULL);
     glutMouseFunc((void(*)(int,int,int,int))NULL);
     glutMotionFunc((void(*)(int,int))NULL);
     glutPassiveMotionFunc((void(*)(int,int))NULL);
