@@ -4,7 +4,7 @@
     created              : Fri Aug 13 22:29:56 CEST 1999
     copyright            : (C) 1999, 2004 by Eric Espie, Bernhard Wymann
     email                : torcs@free.fr
-    version              : $Id: screen.cpp,v 1.15 2005/02/01 15:55:55 berniw Exp $
+    version              : $Id: screen.cpp,v 1.19 2005/08/05 09:26:39 berniw Exp $
 ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file
     Screen management.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: screen.cpp,v 1.15 2005/02/01 15:55:55 berniw Exp $
+    @version	$Id: screen.cpp,v 1.19 2005/08/05 09:26:39 berniw Exp $
     @ingroup	screen
 */
 
@@ -37,8 +37,10 @@
 #endif /* WIN32 */
 
 #include <tgfclient.h>
+#include <portability.h>
 #include "gui.h"
 #include "fg_gm.h"
+#include "glfeatures.h"
 
 //#ifndef WIN32
 //#define USE_RANDR_EXT
@@ -71,7 +73,7 @@ static int usedFG = 0;
 static char	**Res = NULL;
 static int nbRes = 0;
 #else // USE_RANDR_EXT
-static char	*Res[] = {"640x480", "800x600", "1024x768", "1152x864", "1200x960", "1280x1024", "1600x1200", "320x200"};
+static char	*Res[] = {"640x480", "800x600", "1024x768", "1152x768", "1152x864", "1200x854", "1200x960", "1280x1024", "1600x1200", "320x200"};
 static const int nbRes = sizeof(Res) / sizeof(Res[0]);
 #endif // USE_RANDR_EXT
 
@@ -407,6 +409,8 @@ void GfScrInit(int argc, char *argv[])
     GfParmReleaseHandle(handle);
 
     glutReshapeFunc( Reshape );
+
+	checkGLFeatures();
 }
 
 /** Shutdown the screen
@@ -480,8 +484,10 @@ void
 GfScrReinit(void * /* dummy */)
 {
     int retcode = 0;
+	static const int CMDSIZE = 1024;
+	char cmd[CMDSIZE];
+
 #ifndef WIN32
-    char	cmd[1024];
     char	*arg[8];
     int		curArg;
 #endif
@@ -489,7 +495,15 @@ GfScrReinit(void * /* dummy */)
     saveParams();
 
 #ifdef WIN32
-    retcode = execlp("wtorcs.exe", "torcs", (const char *)NULL);
+	snprintf(cmd, CMDSIZE, "%swtorcs.exe", GetLocalDir());
+    int i;
+	for (i = 0; i < CMDSIZE && cmd[i] != NULL; i++) {
+		if (cmd[i] == '/') {
+			cmd[i] = '\\';
+		}
+	}
+	
+	retcode = execlp(cmd, cmd, (const char *)NULL);
 #else
     GfScrShutdown();
 
