@@ -4,7 +4,7 @@
     created              : Sat Mar 18 23:54:30 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: main.cpp,v 1.13 2005/02/01 15:52:14 berniw Exp $
+    version              : $Id: main.cpp,v 1.14 2005/05/27 09:13:17 berniw Exp $
 
  ***************************************************************************/
 
@@ -24,7 +24,6 @@
 
 #include <tgfclient.h>
 #include <client.h>
-#include <policy.h>
 
 #include "linuxspec.h"
 
@@ -110,8 +109,6 @@ main(int argc, char *argv[])
     
     glutMainLoop();		/* event loop of glut */
 
-    DiscretePolicy *dummy = new DiscretePolicy(0,0,0,0,0,0,0);	/* learning lib */
-    
     return 0;			/* just for the compiler, never reached */
 }
 
