@@ -282,10 +282,12 @@ cGrCarCamMirror::~cGrCarCamMirror ()
     delete viewCam;
 }
 
+
 void cGrCarCamMirror::limitFov(void) {
     fovy = 90.0 / screen->getViewRatio();
 }
 
+
 void cGrCarCamMirror::update(tCarElt *car, tSituation * /* s */)
 {
     sgVec3 P, p;
@@ -313,77 +315,81 @@ void cGrCarCamMirror::update(tCarElt *car, tSituation * /* s */)
     up[2] = car->_posMat[2][2];
 }
 
-void cGrCarCamMirror::setViewport (int x, int y, int w, int h)
+
+void cGrCarCamMirror::setViewport(int x, int y, int w, int h)
 {
 	vpx = x;
 	vpy = y;
-	vpw = GfNearestPow2 (w);
-	vph = GfNearestPow2 (h);
-
-	// Create texture object.
-	glBindTexture (GL_TEXTURE_2D, tex);
-	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-	glReadBuffer (GL_BACK);
-
-	glCopyTexImage2D (GL_TEXTURE_2D,
-			0, // map level,
-			GL_RGB, // internal format,
-			vpx, vpy, vpw, vph,
-			0 ); // border
+	vpw = w;
+	vph = h;
 
 	if (viewCam) {
 		delete viewCam;
 	}
-	viewCam = new cGrOrthoCamera (screen, x,  x + w, y, y + h);
-	limitFov ();
+	viewCam = new cGrOrthoCamera(screen, x,  x + w, y, y + h);
+	limitFov();
 }
 
+
 void cGrCarCamMirror::setPos (int x, int y, int w, int h)
 {
-    float dx, dy;
-
     mx = x;
     my = y;
     mw = w;
     mh = h;
 
-    dx = (float)(vpw - w);
-    dy = (float)(vph - h);
-    dx = MAX (0, dx);
-    dy = MAX (0, dy);
-    dx = dx / (float)vpw / 2.0;
-    dy = dy / (float)vph / 2.0;
+    // round up texture size to next power of two
+    tw = GfNearestPow2(w);
+    th = GfNearestPow2(h);
+    if (tw < w) {
+		tw *= 2;
+	}
+    if (th < h) {
+		th *= 2;
+	}
 
-    tsu = 1.0 - dx;
-    tsv = dy;
-    teu = dx;
-    tev = 1.0 - dy;
+    // Create texture object.
+    glBindTexture(GL_TEXTURE_2D, tex);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+    glReadBuffer(GL_BACK);
+
+    glCopyTexImage2D(GL_TEXTURE_2D,
+		      0, // map level,
+		      GL_RGB, // internal format,
+		      0, 0, tw, th,
+		      0 ); // border
+
+    tsu = (float) mw / tw;
+    teu = 0.0;
+    tsv = 0.0;
+    tev = (float) mh / th;
 }
 
 
 void cGrCarCamMirror::activateViewport (void)
 {
-    glViewport (vpx, vpy, vpw, vph);
+    glViewport(vpx, vpy, vpw, vph);
+
+    // Enable scissor test to conserve graphics memory bandwidth.
+    glEnable(GL_SCISSOR_TEST);
+    glScissor(vpx + (vpw - mw)/2, vpy + (vph - mh)/2, mw, mh);
 }
 
 void cGrCarCamMirror::store (void)
 {
-	glBindTexture (GL_TEXTURE_2D, tex);
-	//glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-	//glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-	glReadBuffer (GL_BACK);
+	glDisable(GL_SCISSOR_TEST);
+
+	glBindTexture(GL_TEXTURE_2D, tex);
+	glReadBuffer(GL_BACK);
 
-	/*glCopyTexImage2D (GL_TEXTURE_2D,
-			0, // map level,
-			GL_RGB, // internal format,
-			vpx, vpy, vpw, vph,
-			0 ); // border
-*/
 	// NVidia recommends to NOT use glCopyTexImage2D for performance reasons.
-	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, vpx, vpy, vpw, vph);
+	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0,
+			    vpx + (vpw - mw)/2,
+			    vpy + (vph - mh)/2, mw, mh);
 }
 
+
 void cGrCarCamMirror::display (void)
 {
     viewCam->action ();
@@ -1773,20 +1779,20 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			      id,
 			      1,		/* drawCurr */
 			      0,		/* drawBG  */
-			      74.0,		/* fovy */
-			      1.0,		/* fovymin */
-			      110.0,		/* fovymax */
+			      74.0f,		/* fovy */
+			      1.0f,		/* fovymin */
+			      110.0f,		/* fovymax */
 			      4,		/* up axis */
-			      -grWrldX/2,	/* eyex */
-			      -grWrldY/2,	/* eyey */
-			      0.25 * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
-			      grWrldX/2,	/* centerx */
-			      grWrldY/2,	/* centery */
-			      0,		/* centerz */
-			      10.0,		/* near */
-			      2 * grWrldMaxSize,	/* far */
-			      10 * grWrldMaxSize,	/* fog */
-			      20 * grWrldMaxSize	/* fog */
+			      -grWrldX/2.0f,	/* eyex */
+			      -grWrldY/2.0f,	/* eyey */
+			      0.25f * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
+			      grWrldX/2.0f,	/* centerx */
+			      grWrldY/2.0f,	/* centery */
+			      0.0f,		/* centerz */
+			      10.0f,		/* near */
+			      2.0f * grWrldMaxSize,	/* far */
+			      10.0f * grWrldMaxSize,	/* fog */
+			      20.0f * grWrldMaxSize	/* fog */
 			      );
     cam->add(&cams[c]);
     id++;
@@ -1796,20 +1802,20 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			      id,
 			      1,		/* drawCurr */
 			      0,		/* drawBG  */
-			      74.0,		/* fovy */
-			      1.0,		/* fovymin */
-			      110.0,		/* fovymax */
+			      74.0f,		/* fovy */
+			      1.0f,		/* fovymin */
+			      110.0f,		/* fovymax */
 			      4,		/* up axis */
-			      -grWrldX/2,	/* eyex */
-			      grWrldY * 3/2,	/* eyey */
-			      0.25 * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
-			      grWrldX/2,	/* centerx */
-			      grWrldY/2,	/* centery */
-			      0,		/* centerz */
-			      10.0,		/* near */
-			      2 * grWrldMaxSize,	/* far */
-			      10 * grWrldMaxSize,	/* fog */
-			      20 * grWrldMaxSize	/* fog */
+			      -grWrldX/2.0f,	/* eyex */
+			      grWrldY * 3.0f/2.0f,	/* eyey */
+			      0.25f * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
+			      grWrldX/2.0f,	/* centerx */
+			      grWrldY/2.0f,	/* centery */
+			      0.0f,		/* centerz */
+			      10.0f,		/* near */
+			      2.0f * grWrldMaxSize,	/* far */
+			      10.0f * grWrldMaxSize,	/* fog */
+			      20.0f * grWrldMaxSize	/* fog */
 			      );
     cam->add(&cams[c]);
     id++;
@@ -1819,20 +1825,20 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			      id,
 			      1,		/* drawCurr */
 			      0,		/* drawBG  */
-			      74.0,		/* fovy */
-			      1.0,		/* fovymin */
-			      110.0,		/* fovymax */
+			      74.0f,		/* fovy */
+			      1.0f,		/* fovymin */
+			      110.0f,		/* fovymax */
 			      4,		/* up axis */
-			      grWrldX * 3/2,	/* eyex */
-			      grWrldY * 3/2,	/* eyey */
-			      0.25 * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
-			      grWrldX/2,	/* centerx */
-			      grWrldY/2,	/* centery */
-			      0,		/* centerz */
-			      10.0,		/* near */
-			      2 * grWrldMaxSize,	/* far */
-			      10 * grWrldMaxSize,	/* fog */
-			      20 * grWrldMaxSize	/* fog */
+			      grWrldX * 3.0f/2.0f,	/* eyex */
+			      grWrldY * 3.0f/2.0f,	/* eyey */
+			      0.25f * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
+			      grWrldX/2.0f,	/* centerx */
+			      grWrldY/2.0f,	/* centery */
+			      0.0f,		/* centerz */
+			      10.0f,		/* near */
+			      2.0f * grWrldMaxSize,	/* far */
+			      10.0f * grWrldMaxSize,	/* fog */
+			      20.0f * grWrldMaxSize	/* fog */
 			      );
     cam->add(&cams[c]);
     id++;
@@ -1842,20 +1848,20 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			      id,
 			      1,		/* drawCurr */
 			      0,		/* drawBG  */
-			      74.0,		/* fovy */
-			      1.0,		/* fovymin */
-			      110.0,		/* fovymax */
+			      74.0f,		/* fovy */
+			      1.0f,		/* fovymin */
+			      110.0f,		/* fovymax */
 			      4,		/* up axis */
-			      grWrldX * 3/2,	/* eyex */
-			      -grWrldY/2,	/* eyey */
-			      0.25 * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
-			      grWrldX/2,	/* centerx */
-			      grWrldY/2,	/* centery */
-			      0,		/* centerz */
-			      10.0,		/* near */
-			      2 * grWrldMaxSize,	/* far */
-			      10 * grWrldMaxSize,	/* fog */
-			      20 * grWrldMaxSize	/* fog */
+			      grWrldX * 3.0f/2.0f,	/* eyex */
+			      -grWrldY/2.0f,	/* eyey */
+			      0.25f * sqrt(grWrldX*grWrldX+grWrldY*grWrldY), /* eyez */
+			      grWrldX/2.0f,	/* centerx */
+			      grWrldY/2.0f,	/* centery */
+			      0.0f,		/* centerz */
+			      10.0f,		/* near */
+			      2.0f * grWrldMaxSize,	/* far */
+			      10.0f * grWrldMaxSize,	/* fog */
+			      20.0f * grWrldMaxSize	/* fog */
 			      );
     cam->add(&cams[c]);
     id++;
