@@ -42,14 +42,15 @@
 #include "grutil.h"
 #include "grtrackmap.h"
 #include "grcarlight.h"
+#include <glfeatures.h>
 
 int maxTextureUnits = 0;
-static double	OldTime;
-static int	nFrame;
-float		grFps;
-double		grCurTime;
-double		grDeltaTime;
-int		segIndice	= 0;
+static double OldTime;
+static int nFrame;
+float grFps;
+double grCurTime;
+double grDeltaTime;
+int segIndice = 0;
 
 tdble grMaxDammage = 10000.0;
 int grNbCars = 0;
@@ -61,68 +62,56 @@ int grWinx, grWiny, grWinw, grWinh;
 
 static float grMouseRatioX, grMouseRatioY;
 
-tgrCarInfo	*grCarInfo;
-ssgContext	grContext;
-
-
+tgrCarInfo *grCarInfo;
+ssgContext grContext;
 class cGrScreen *grScreens[GR_NB_MAX_SCREEN] = {NULL, NULL, NULL, NULL};
-
 int grNbScreen = 1;
-
-tdble	grLodFactorValue = 1.0;
+tdble grLodFactorValue = 1.0;
 
 
 static char buf[1024];
 
 #ifdef WIN32
-#include "win32_glext.h"
+#include <GL/glext.h>
 PFNGLMULTITEXCOORD2FARBPROC glMultiTexCoord2fARB = NULL;
-PFNGLMULTITEXCOORD2FVARBPROC glMultiTexCoord2fvARB =NULL;
-PFNGLACTIVETEXTUREARBPROC   glActiveTextureARB = NULL;
+PFNGLMULTITEXCOORD2FVARBPROC glMultiTexCoord2fvARB = NULL;
+PFNGLACTIVETEXTUREARBPROC glActiveTextureARB = NULL;
 PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB = NULL;
 #endif
 
+
 // InitMultiTex
 // desc: sets up OpenGL for multitexturing support
 bool InitMultiTex(void)
 {
-    if (GetSingleTextureMode ()) {
-	maxTextureUnits = 1;
-	return true;
-    } else {
-
-	char *extensionStr;			// list of available extensions
-		
-	extensionStr = (char*)glGetString(GL_EXTENSIONS);
-
-	if (extensionStr == NULL)
-		return false;
-
-	/* printf("glextensionstr: %s\n", extensionStr); */
-
-	if (strstr(extensionStr, "GL_ARB_multitexture"))
-	{
-		// retrieve the maximum number of texture units allowed
-		glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &maxTextureUnits);
-		/* printf("\nfound number of texture units : %d\n", maxTextureUnits); */
-#ifdef WIN32
-		// retrieve addresses of multitexturing functions
-		glMultiTexCoord2fARB = (PFNGLMULTITEXCOORD2FARBPROC) wglGetProcAddress("glMultiTexCoord2fARB");
-		glActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC) wglGetProcAddress("glActiveTextureARB");
-		glClientActiveTextureARB = (PFNGLCLIENTACTIVETEXTUREARBPROC) wglGetProcAddress("glClientActiveTextureARB");
-		glMultiTexCoord2fvARB = (PFNGLMULTITEXCOORD2FVARBPROC) wglGetProcAddress("glMultiTexCoord2fvARB");
-#endif
+	if (GetSingleTextureMode ()) {
+		maxTextureUnits = 1;
 		return true;
-	}
-	else
-		return false;
-	
+    } else {
+		// list of available extensions
+		char *extensionStr = (char*)glGetString(GL_EXTENSIONS);
+		if (extensionStr == NULL)
+			return false;
+
+		if (strstr(extensionStr, "GL_ARB_multitexture")) {
+			// retrieve the maximum number of texture units allowed
+			glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &maxTextureUnits);
+#ifdef WIN32
+			// retrieve addresses of multitexturing functions
+			glMultiTexCoord2fARB = (PFNGLMULTITEXCOORD2FARBPROC) wglGetProcAddress("glMultiTexCoord2fARB");
+			glActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC) wglGetProcAddress("glActiveTextureARB");
+			glClientActiveTextureARB = (PFNGLCLIENTACTIVETEXTUREARBPROC) wglGetProcAddress("glClientActiveTextureARB");
+			glMultiTexCoord2fvARB = (PFNGLMULTITEXCOORD2FVARBPROC) wglGetProcAddress("glMultiTexCoord2fvARB");
+#endif
+			return true;
+		} else {
+			return false;
+		}
     }
 }
 
 
-static void
-grAdaptScreenSize(void)
+static void grAdaptScreenSize(void)
 {
     switch (grNbScreen) {
     case 0:
@@ -156,7 +145,7 @@ grAdaptScreenSize(void)
 static void
 grSplitScreen(void *vp)
 {
-    int p = (int)vp;
+    long p = (long)vp;
 
     switch (p) {
     case GR_SPLIT_ADD:
@@ -198,19 +187,19 @@ grGetcurrentScreen(void)
 static void
 grSetZoom(void *vp)
 {
-    grGetcurrentScreen()->setZoom((int)vp);
+    grGetcurrentScreen()->setZoom((long)vp);
 }
 
 static void
 grSelectCamera(void *vp)
 {
-    grGetcurrentScreen()->selectCamera((int)vp);
+    grGetcurrentScreen()->selectCamera((long)vp);
 }
 
 static void
 grSelectBoard(void *vp)
 {
-    grGetcurrentScreen()->selectBoard((int)vp);
+    grGetcurrentScreen()->selectBoard((long)vp);
 }
 
 static void
@@ -244,7 +233,7 @@ initView(int x, int y, int width, int height, int /* flag */, void *screen)
 
     if (maxTextureUnits==0)
       {
-	InitMultiTex();    
+	InitMultiTex();
       }
     
     grWinx = x;
@@ -408,9 +397,6 @@ initCars(tSituation *s)
 
     grAdaptScreenSize();
 
-    //int nb = grPruneTree(TheScene, true);
-    //GfOut("PRUNE SSG TREE: removed %d empty branches\n", nb);
-
     return 0;
     
 }
@@ -438,12 +424,7 @@ shutdownCars(void)
 			}
 		}
 
-		/* for (i = 0; i < grNbCars; i++) { */
-		/*     CarsAnchor->removeKid(grCarInfo[i].carTransform); */
-		/*     ShadowAnchor->removeKid(grCarInfo[i].shadowAnchor); */
-		/* } */
 		PitsAnchor->removeAllKids();
-		/* if (grTrack->pits.type == TR_PIT_ON_TRACK_SIDE) PitsAnchor->removeKid(ThePits); */
 		ThePits = 0;
 		free(grCarInfo);
 	}
@@ -460,7 +441,10 @@ initTrack(tTrack *track)
 {
 	int i;
 
+	// The inittrack does as well init the context, that is highly inconsistent, IMHO.
+	// TODO: Find a solution to init the graphics first independent of objects.
 	grContext.makeCurrent();
+
 	grTrackHandle = GfParmReadFile(track->filename, GFPARM_RMODE_STD | GFPARM_RMODE_CREAT);
 	grLoadScene(track);
 
@@ -486,7 +470,6 @@ shutdownTrack(void)
 			grScreens[i] = NULL;
 		}
 	}
-
 }
 
 /*void bendCar (int index, sgVec3 poc, sgVec3 force, int cnt)
