@@ -24,35 +24,17 @@
 #include <plib/ssg.h>
 #include <raceman.h>
 
-#if GL_VERSION_1_3
-#define glActiveTextureARB glActiveTexture
-#define glMultiTexCoord2fvARB glMultiTexCoord2fv
-#define glClientActiveTextureARB glClientActiveTexture
-#endif
-
 #ifdef WIN32
 #include <windows.h>
 #include <GL/gl.h>
-#include "win32_glext.h"
+#include <GL/glext.h>
+
 ////// Multitexturing Info
-/*typedef void (APIENTRY * PFNGLMULTITEXCOORD2FARBPROC) (GLenum target, GLfloat s, GLfloat t);
-typedef void (APIENTRY * PFNGLACTIVETEXTUREARBPROC) (GLenum texture);
-typedef void (APIENTRY * PFNGLCLIENTACTIVETEXTUREARBPROC) (GLenum texture);*/
 extern PFNGLMULTITEXCOORD2FARBPROC glMultiTexCoord2fARB ;
 extern PFNGLMULTITEXCOORD2FVARBPROC glMultiTexCoord2fvARB;
 extern PFNGLACTIVETEXTUREARBPROC   glActiveTextureARB ;
 extern PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB ;
 
-/*
-  PFNGLMULTITEXCOORD2FVARBPROC	glMultiTexCoord2fvARB		= NULL;
-  PFNGLACTIVETEXTUREARBPROC		glActiveTextureARB			= NULL;
-  PFNGLCLIENTACTIVETEXTUREARBPROC	glClientActiveTextureARB	= NULL;
-  
-  PFNGLDRAWARRAYSEXTPROC			glDrawArraysEXT				= NULL;
-  PFNGLVERTEXPOINTEREXTPROC		glVertexPointerEXT			= NULL;
-  PFNGLNORMALPOINTEREXTPROC		glNormalPointerEXT			= NULL;
-  PFNGLTEXCOORDPOINTEREXTPROC		glTexCoordPointerEXT		= NULL;
-*/
 extern bool InStr(char *searchStr, char *str);
 #endif
 
