@@ -4,7 +4,7 @@
     created              : Sun Mar 19 00:05:43 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: car.cpp,v 1.18 2005/02/01 18:54:02 berniw Exp $
+    version              : $Id: car.cpp,v 1.22 2005/05/18 19:33:27 berniw Exp $
 
  ***************************************************************************/
 
@@ -176,7 +176,9 @@ SimCarUpdateForces(tCar *car)
 	/* moments */
 	F.M.x += car->wheel[i].forces.z * car->wheel[i].staticPos.y +
 	    car->wheel[i].forces.y * car->wheel[i].rollCenter;
-	F.M.y -= car->wheel[i].forces.z * car->wheel[i].staticPos.x + 
+		// Eventually TODO: activate fix below and make all cars/robots fit.
+		//car->wheel[i].forces.y * (car->statGC.z + car->wheel[i].rideHeight);
+	F.M.y -= car->wheel[i].forces.z * car->wheel[i].staticPos.x +
 	    car->wheel[i].forces.x * (car->statGC.z + car->wheel[i].rideHeight);
 	F.M.z += -car->wheel[i].forces.x * car->wheel[i].staticPos.y +
 	    car->wheel[i].forces.y * car->wheel[i].staticPos.x;
@@ -184,7 +186,7 @@ SimCarUpdateForces(tCar *car)
 
     /* Aero Drag */
     F.F.x += car->aero.drag;
-    
+
     /* Wings & Aero Downforce */
     for (i = 0; i < 2; i++) {
 	/* forces */
@@ -258,13 +260,13 @@ SimCarUpdateSpeed(tCar *car)
     Rm = Rr * car->wheelbase /*  / 2.0 */ * car->Iinv.z * SimDeltaTime;
     Rr = 2.0 * Rr / mass * SimDeltaTime;
     vel = sqrt(car->DynGCg.vel.x * car->DynGCg.vel.x + car->DynGCg.vel.y * car->DynGCg.vel.y);
-    
+
     if (Rr > vel) {
 	Rr = vel;
     }
     if (vel > 0.00001) {
-	car->DynGCg.vel.x -= car->DynGCg.vel.x * Rr / vel * SIGN(car->DynGCg.vel.x);
-	car->DynGCg.vel.y -= car->DynGCg.vel.y * Rr / vel * SIGN(car->DynGCg.vel.y);
+		car->DynGCg.vel.x -= car->DynGCg.vel.x * Rr / vel * SIGN(car->DynGCg.vel.x);
+		car->DynGCg.vel.y -= car->DynGCg.vel.y * Rr / vel * SIGN(car->DynGCg.vel.y);
     }
 
     car->DynGCg.vel.ax += car->DynGCg.acc.ax * SimDeltaTime;
@@ -291,6 +293,8 @@ SimCarUpdateSpeed(tCar *car)
     car->DynGC.vel.y = -car->DynGCg.vel.x * Sinz + car->DynGCg.vel.y * Cosz;
     car->DynGC.vel.z = car->DynGCg.vel.z;
 
+	//printf ("%f\n", car->DynGC.vel.az);
+
 }
 
 void
@@ -391,8 +395,27 @@ SimCarUpdateCornerPos(tCar *car)
 	/* the speed is vel.az * r                   */
 	/* where r = sqrt(x*x + y*y)                 */
 	/* the tangent vector is -y / r and x / r    */
+	// Hm, here local and global frames are mixed.
+	if (0) {
 	car->corner[i].vel.x = vx - car->DynGCg.vel.az * y;
 	car->corner[i].vel.y = vy + car->DynGCg.vel.az * x;
+	} else {
+	// This bit should clean it up.
+	// compute corner velocity at local frame
+	car->corner[i].vel.ax = - car->DynGC.vel.az * y;
+	car->corner[i].vel.ay = car->DynGC.vel.az * x;
+  	
+	// rotate to global and add global center of mass velocity
+	// note: global to local.
+	car->corner[i].vel.x = vx
+		+ car->corner[i].vel.ax * Cosz - car->corner[i].vel.ay * Sinz;
+	car->corner[i].vel.y = vy
+		+ car->corner[i].vel.ax * Sinz + car->corner[i].vel.ay * Cosz;
+
+	// add local center of mass velocity
+	car->corner[i].vel.ax += car->DynGC.vel.x;
+	car->corner[i].vel.ay += car->DynGC.vel.y;
+	}
     }
 }
 
