@@ -4,7 +4,7 @@
     created      : Sun Dec 24 12:14:18 CET 2000
     copyright    : (C) 2000 by Eric Espie
     email        : eric.espie@torcs.org
-    version      : $Id: track.cpp,v 1.20 2003/05/18 20:41:33 torcs Exp $
+    version      : $Id: track.cpp,v 1.21 2005/07/19 19:55:56 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file   
     		
     @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
-    @version	$Id: track.cpp,v 1.20 2003/05/18 20:41:33 torcs Exp $
+    @version	$Id: track.cpp,v 1.21 2005/07/19 19:55:56 berniw Exp $
 */
 
 #include <stdio.h>
@@ -432,7 +432,8 @@ InitScene(tTrack *Track, void *TrackHandle, int bump)
     nbvert *= 30;
     nbvert+=58; /* start bridge */
     nbvert+=12 + 10 * Track->pits.driversPitsNb;
-    nbvert+=1000; /* safety margin */
+	// TODO: Investigate if this is a feasible solution, nbvert was perviously += 1000.
+    nbvert+=10000; /* safety margin */
     printf("=== Indices Array Size   = %d\n", nbvert);
     printf("=== Vertex Array Size    = %d\n", nbvert * 3);
     printf("=== Tex Coord Array Size = %d\n", nbvert * 2);
@@ -2582,6 +2583,21 @@ SaveMainTrack(FILE *curFd, int bump)
 }
 
 
+/** Calculate track parameters and exit without any file creation
+    It is for information only, mainly for use from TrackEditor.
+    @param	Track	track structure
+    @param	TrackHandle	handle on the track description
+    @return	none
+*/
+void CalculateTrack(tTrack * Track, void *TrackHandle, int bump)
+{
+	TrackStep = GfParmGetNum(TrackHandle, TRK_SECT_TERRAIN, TRK_ATT_TSTEP, NULL, TrackStep);
+	GfOut("Track step: %.2f ", TrackStep);
+	InitScene(Track, TrackHandle, bump);
+	printf("Calculation finished\n");
+}
+
+
 /** Generate the track AC3D file(s).
     @param	Track	track structure
     @param	TrackHandle	handle on the track description
